/************************************************************************
 *
 *    blkshft.c  --  intelligent block shift routine
 */

#include "cbtree.h"

/*******************************/

void blkshft(dstp, srcp, num, blksz)

/*
 *   shift n pages from src to dst, considers overlapping memory regions.
 *
 */
char *dstp;          /* destination address */
char *srcp;          /* source address      */
uword num;           /* # blocks to copy    */
uword blksz;         /* block size          */
{
   uword n = num * blksz;

   /* FORWARD COPY (preferred) */
   /*   src:     <--  |---------------------|   */
   /*   dst:   | - - - - - - - - - - |          */

   if (dstp < srcp ||                  /* either we're copying to lower */
         srcp + n <= dstp)             /* memory or we're NOT overlapping */

      memcpy(dstp, srcp, n); /* copy upwards */

   /* REVERSE BLOCK COPY (slightly slower) */
   /*   src:   |---------------------|  -->     */
   /*   dst:          | - - - - - - - - - - |   */

   else  /* ranges are overlapping AND we're copy to higher memory */
   {
      srcp += n;
      dstp += n;
      while (num-- != 0)
         memcpy(dstp -= blksz, srcp -= blksz, blksz); /* a blk at a time */
   }
}
